#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>


#include "../sshguard_log.h"
#include "../sshguard_fw.h"
#include "command.h"

static int run_command(char *command, char *ip);
static int getcommand(const char *cmdformat, const char *ip, char **command, char *command_args_ar[]);

int fw_init() {
    char *cmds[] = COMMANDS_INIT;
    int i, ret;

    for (i = 0, ret = 0; i < sizeof(cmds)/sizeof(char*) && !ret; i++) {
        ret = run_command(cmds[i], NULL);
    }
    return ret;
}

int fw_fin() {
    char *cmds[] = COMMANDS_FIN;
    int i, ret;

    for (i = 0, ret = 0; i < sizeof(cmds)/sizeof(char*) && !ret; i++) {
        ret = run_command(cmds[i], NULL);
    }
    return ret;
}

int block_ip(char *ip) {
    char *cmds[] = COMMANDS_BLOCK;
    int i, ret;

    for (i = 0, ret = 0; i < sizeof(cmds)/sizeof(char*) && !ret; i++) {
        ret = run_command(cmds[i], ip);
    }
    return ret;
}

int release_ip(char *ip) {
    char *cmds[] = COMMANDS_RELEASE;
    int i, ret;

    for (i = 0, ret = 0; i < sizeof(cmds)/sizeof(char*) && !ret; i++) {
        ret = run_command(cmds[i], ip);
    }
    return ret;
}

int flush_blocked(void) {
    char *cmds[] = COMMANDS_FLUSH;
    int i, ret;

    for (i = 0, ret = 0; i < sizeof(cmds)/sizeof(char*) && !ret; i++) {
        ret = run_command(cmds[i], "");
    }
    return ret;
}

static int run_command(char *command, char *ip) {
    pid_t child;
    char *cmd = NULL;
    int result;
    char *cmdargs[MAXCMDOPTS];

    /* nothing to run? */
    if (command == NULL) return 0;

    result = getcommand(command, ip, &cmd, cmdargs);
    if (result <= 0) {
        sshguard_log(LOG_ERR, "Unable to parse command.");
        return -1;
    }

    sshguard_log(LOG_DEBUG, "Running command \"%s\"", cmd);
    
    child = fork();
    if (child < 0) {
        sshguard_log(LOG_ERR, "Unable to fork(): %s", strerror(errno));
        return -1;
    }

    if (child == 0) { 
        /* child */
        close(0); close(1);  close(2);
        execv(cmdargs[0], cmdargs);
        sshguard_log(LOG_ERR, "Unable to exec(): %s", strerror(errno));
        exit(1);
    }
    waitpid(child, &result, 0);
    
    free(cmd);

    return WEXITSTATUS(result);
}

static int getcommand(const char *cmdformat, const char *ip, char **command, char *command_args_ar[]) {
    char *cmd;
    int parsestat, i, optnum;
    

    if (strstr(cmdformat, "%s") != NULL) {
        /* requires IP replacement */
        cmd = malloc(strlen(cmdformat) - 2 + strlen(ip) + 1);
        sprintf(cmd, cmdformat, ip);
    } else {
        cmd = malloc(strlen(cmdformat)+1);
        strcpy(cmd, cmdformat);
    }

    /* parse command */
    parsestat = 0;
    for (i = optnum = 0; cmd[i] != '\0' && optnum < MAXCMDOPTS; i++) {
        if (parsestat) {
            /* got chars so far */
            if (isspace(cmd[i])) {
                cmd[i] = '\0';
                parsestat = 0;
                optnum++;
            }
        } else {
            /* got blanks so far */
            if (! isspace(cmd[i])) {
                command_args_ar[optnum] = &cmd[i];
                parsestat = 1;
            }
        }
    }
    if (optnum == MAXCMDOPTS) {
        sshguard_log(LOG_ERR, "Command too complex. Increase MAXCMDOPTS in sshguard_fw sources.");
        return -1;
    }
    if (parsestat) optnum++;
    command_args_ar[optnum] = NULL;

    *command = cmd;

    return optnum;
}

