
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <simclist.h>

#include "../sshguard_log.h"
#include "../sshguard_fw.h"

#define IPFW_RULERANGE_MIN      55000
#define IPFW_RULERANGE_MAX      55050

#define MAXIPFWCMDLEN           60

struct ip_ruleno_s {
    char ip[16];
    unsigned int ruleno;
};

list_t iprulenumbers;


void ipfwmod_logsystemretval(char *command, int returnval);


int fw_init() {
    srandom(time(NULL));
    list_init(&iprulenumbers);
    return 0;
}


int fw_fin() {
    return 0;
}

int block_ip(char *ip) {
    unsigned int ruleno;
    int ret;
    char command[MAXIPFWCMDLEN];

    ruleno = (random() % (IPFW_RULERANGE_MAX - IPFW_RULERANGE_MIN)) + IPFW_RULERANGE_MIN;
    sprintf(command, "ipfw add %u drop ip from %s to me", ruleno, ip);

    ret = system(command);
    if (ret == 0) { /* success, save rule number */
        struct ip_ruleno_s *addendum = malloc(sizeof(struct ip_ruleno_s));

        strcpy(addendum->ip, ip);
        addendum->ruleno = ruleno;

        list_append(&iprulenumbers, addendum);
    }
    ipfwmod_logsystemretval(command, ret);
    
    return ret;
}

int release_ip(char *ip) {
    struct ip_ruleno_s *data = NULL;
    char command[MAXIPFWCMDLEN];
    int pos, ret = 0;

    list_iterator_start(&iprulenumbers);
    for (pos = 0; list_iterator_hasnext(&iprulenumbers); pos++) {
        data = (struct ip_ruleno_s *)list_iterator_next(&iprulenumbers);
        if (strcmp(data->ip, ip) == 0) /* found */
            break;
    }
    list_iterator_stop(&iprulenumbers);

    snprintf(command, MAXIPFWCMDLEN, "ipfw delete %u", data->ruleno);
    ret = system(command);
    if (ret == 0) {
        list_delete_at(&iprulenumbers, pos);
        free(data);
    }
    ipfwmod_logsystemretval(command, ret);

    return ret;
}

int flush_blocked() {
    struct ip_ruleno_s *data;
    char command[MAXIPFWCMDLEN];
    int ret;

    list_iterator_start(&iprulenumbers);
    while (list_iterator_hasnext(&iprulenumbers)) {
        data = (struct ip_ruleno_s *)list_iterator_next(& iprulenumbers);
        snprintf(command, MAXIPFWCMDLEN, "ipfw del %u", data->ruleno);
        ret = system(command);
        ipfwmod_logsystemretval(command, ret);
    }
    list_iterator_stop(& iprulenumbers);
    
    list_clear(&iprulenumbers);

    return 0;
}

void ipfwmod_logsystemretval(char *command, int returnval) {
    if (returnval < 0) {
        sshguard_log(LOG_ERR, "Unable to execute command \"%s\". Exited %d", command, returnval);
    } else if (returnval == 127) {
        sshguard_log(LOG_ERR, "Unable to execute the shell.");
    }
}

